@@ -1,8 +1,7 @@
 from inferelator.distributed import AbstractController
 from inferelator import utils
-import warnings
 
-DEFAULT_MP_ENGINE = "local"
+DEFAULT_MP_ENGINE = "joblib"
 
 
 class MPControl(AbstractController):
@@ -14,9 +13,6 @@ class MPControl(AbstractController):
     _controller_name = "multiprocessing_registry"
     client = None
 
-    # Relevant external state booleans
-    is_initialized = False
-
     @classmethod
     def name(cls):
         """
@@ -33,7 +29,7 @@ class MPControl(AbstractController):
         """
         if cls.client is None:
             return False
-        return cls.client.is_dask()
+        return cls.client._controller_dask
 
     @classmethod
     def set_multiprocess_engine(cls, engine, processes=None):
@@ -46,6 +42,7 @@ class MPControl(AbstractController):
         dask-k8
         dask-local
         multiprocessing
+        joblib
         local
 
         :param engine: A string to lookup the controller or a Controller object
@@ -53,11 +50,22 @@ class MPControl(AbstractController):
         :param processes: Number of processes to use. Equivalent to calling `set_processes`
         :type processes: int
         """
-        if cls.is_initialized:
-            raise RuntimeError("Client is currently active. Run .shutdown() before changing engines.")
 
         if utils.is_string(engine):
-            if engine == "dask-cluster":
+            engine = engine.lower()
+
+            # If this is already the selected engine, move on
+            if cls.client is not None and cls.client.name() == engine:
+                pass
+            elif cls.client is not None and cls.client.name() == 'joblib' and engine == 'multiprocessing':
+                pass
+
+            # Check to see if there's something running that has to be handled
+            elif cls.client is not None and cls.status() and cls.client._require_shutdown:
+                raise RuntimeError("Client is currently active. Run .shutdown() before changing engines.")
+
+            # Dask engines
+            elif engine == "dask-cluster":
                 from inferelator.distributed.dask_cluster_controller import DaskHPCClusterController
                 cls.client = DaskHPCClusterController
             elif engine == "dask-local":
@@ -66,22 +74,28 @@ class MPControl(AbstractController):
             elif engine == "dask-k8":
                 from inferelator.distributed.dask_k8_controller import DaskK8Controller
                 cls.client = DaskK8Controller
+
+            # Old, dead key-value storage server engine
             elif engine == "kvs":
                 raise DeprecationWarning("The KVS engine is deprecated. Use Dask-based multiprocessing")
-            elif engine == "multiprocessing":
-                from inferelator.distributed.multiprocessing_controller import MultiprocessingController
-                cls.client = MultiprocessingController
+
+            # Local engines
+            elif engine == "multiprocessing" or engine == "joblib":
+                from inferelator.distributed.joblib_controller import JoblibController
+                cls.client = JoblibController
             elif engine == "local":
                 from inferelator.distributed.local_controller import LocalController
                 cls.client = LocalController
             else:
-                raise ValueError("Engine {eng_str} unknown".format(eng_str=engine))
+                raise ValueError(f"Engine {engine} is not a valid argument")
+
         elif issubclass(engine, AbstractController):
             cls.client = engine
+
         else:
             raise ValueError("Engine must be provided as a string for lookup or an implemented Controller class object")
 
-        utils.Debug.vprint("Inferelator MPControl using engine {eng}".format(eng=cls.name()))
+        utils.Debug.vprint("Inferelator parallelized using engine {eng}".format(eng=cls.name()))
 
         if processes is not None:
             cls.set_processes(processes)
@@ -92,27 +106,25 @@ class MPControl(AbstractController):
         Connect to the manager or scheduler or process pool or whatever using the `.connect()` implementation in the
         multiprocessing engine.
         """
-        if cls.is_initialized:
-            return True
 
         if cls.client is None:
-            utils.Debug.vprint("Loading default engine {eng}".format(eng=DEFAULT_MP_ENGINE))
+            utils.Debug.vprint(
+                f"Loading default engine {DEFAULT_MP_ENGINE}"
+            )
+
             cls.set_multiprocess_engine(DEFAULT_MP_ENGINE)
 
-        connect_return = cls.client.connect(*args, **kwargs)
-
-        # Set the process state
-        cls.is_initialized = True
-
-        return connect_return
+        return cls.client.connect(*args, **kwargs)
 
     @classmethod
     def map(cls, func, *args, **kwargs):
         """
         Map using the `.map()` implementation in the multiprocessing engine
         """
-        if not cls.is_initialized:
-            raise RuntimeError("Connect before calling map()")
+
+        if cls.client is None:
+            cls.connect()
+
         return cls.client.map(func, *args, **kwargs)
 
     @classmethod
@@ -120,8 +132,10 @@ class MPControl(AbstractController):
         """
         Set worker process count
         """
-        if cls.is_initialized:
-            raise RuntimeError("Cannot set processes after the engine has started")
+
+        if cls.client is None:
+            cls.connect()
+
         return cls.client.set_processes(process_count)
 
     @classmethod
@@ -130,11 +144,24 @@ class MPControl(AbstractController):
         Gracefully shut down the multiprocessing engine by calling `.shutdown()`
         """
 
-        if cls.is_initialized:
-            client_off = cls.client.shutdown()
-            cls.is_initialized = False
-            cls.client = None
+        if cls.client is not None:
+            return cls.client.shutdown()
         else:
-            client_off = True
+            return None
 
-        return client_off
+    @classmethod
+    def status(cls):
+        """
+        True if ready to process data, False otherwise
+        """
+
+        if cls.client is None:
+            return False
+        elif not cls.client._require_initialization:
+            return True
+        elif hasattr(cls.client, 'client') and cls.client.client is not None:
+            return True
+        elif hasattr(cls.client, 'client') and cls.client.client is None:
+            return False
+        else:
+            return False
