@@ -263,6 +263,22 @@ class InferelatorData(object):
         else:
             return self._adata.X.nbytes
 
+    @property
+    def prior_data(self):
+        return self._adata.uns["prior_data"] if "prior_data" in self._adata.uns else None
+
+    @prior_data.setter
+    def prior_data(self, new_prior):
+        self._adata.uns["prior_data"] = new_prior
+
+    @property
+    def tfa_prior_data(self):
+        return self._adata.uns["tfa_prior_data"] if "tfa_prior_data" in self._adata.uns else None
+
+    @tfa_prior_data.setter
+    def tfa_prior_data(self, new_prior):
+        self._adata.uns["tfa_prior_data"] = new_prior
+
     @property
     def meta_data(self):
         return self._adata.obs
@@ -531,7 +547,9 @@ class InferelatorData(object):
             nz_var = nz_var.A.flatten() if self.is_sparse else nz_var
 
             if np.any(np.isnan(nz_var)):
-                raise ValueError("NaN values are present in the expression matrix; unable to remove var=0 genes")
+                raise ValueError(
+                    f"NaN values are present in data matrix {self.name} "
+                    f"when removing zero variance genes")
 
             nz_var = comp < nz_var
 
@@ -541,23 +559,31 @@ class InferelatorData(object):
             var_zero_trim = 0
 
         if np.sum(keep_column_bool) == 0:
-            err_msg = "No genes remain after trimming. ({lst} removed to match list, {v} removed for var=0)"
-            raise ValueError(err_msg.format(lst=list_trim, v=var_zero_trim))
+            raise ValueError(
+                "No genes remain after trimming. "
+                f"({list_trim} removed to match list, "
+                f"{var_zero_trim} removed for zero variance)"
+            )
 
         if np.sum(keep_column_bool) == self._adata.shape[1]:
             pass
         else:
-            Debug.vprint("Trimming {name} matrix {sh} to {n} columns".format(name=self.name, sh=self._adata.X.shape,
-                                                                             n=np.sum(keep_column_bool)),
-                         level=1)
+            Debug.vprint(
+                f"Trimming {self.name} matrix "
+                f"{self._adata.X.shape} to "
+                f"{np.sum(keep_column_bool)} columns",
+                level=1
+            )
 
             # This explicit copy allows the original to be deallocated
-            # Otherwise the GC leaves the original because the view reference keeps it alive
+            # Otherwise the view reference keeps it in memory
             # At some point it will need to copy so why not now
-            self._adata = AnnData(self._adata.X[:, keep_column_bool],
-                                  obs=self._adata.obs.copy(),
-                                  var=self._adata.var.loc[keep_column_bool, :].copy(),
-                                  dtype=self._adata.X.dtype)
+            self._adata = AnnData(
+                self._adata.X[:, keep_column_bool],
+                obs=self._adata.obs.copy(),
+                var=self._adata.var.loc[keep_column_bool, :].copy(),
+                dtype=self._adata.X.dtype
+            )
 
             # Make sure that there's no hanging reference to the original object
             gc.collect()
@@ -574,7 +600,7 @@ class InferelatorData(object):
             x = x.X
 
         if zscore:
-            
+
             # Z-score the values
             z_x = np.subtract(x, self.obs_means.reshape(-1, 1))
             z_x = np.divide(z_x, self.obs_stdev.reshape(-1, 1))
@@ -628,7 +654,7 @@ class InferelatorData(object):
         :param random_seed: Seed for numpy random generator, defaults to None. Will be ignored if a generator itself is
             passed to random_gen.
         :type random_seed: int, optional
-        :param random_gen: Numpy random generator to use, defaults to None. 
+        :param random_gen: Numpy random generator to use, defaults to None.
         :type random_gen: np.random.Generator, optional
         :param inplace: Change this instance of the data structure inplace and return a reference to itself
         :type inplace: bool, optional
@@ -648,7 +674,7 @@ class InferelatorData(object):
         # Sample with replacement using randint
         if with_replacement:
             keeper_ilocs = random_gen.integers(self.num_obs, size=(num_obs,))
-        
+
         # Sample without replacement using choice
         else:
             keeper_ilocs = random_gen.choice(np.arange(self.num_obs), size=(num_obs,), replace=False)
@@ -657,7 +683,7 @@ class InferelatorData(object):
         if inplace:
             self._adata = self._adata[keeper_ilocs, :].copy()
             return_obj = self
-        
+
         # Create a new InferelatorData instance with the _adata slice
         else:
             return_obj = InferelatorData(self._adata[keeper_ilocs, :].copy())
@@ -667,7 +693,6 @@ class InferelatorData(object):
 
         return return_obj
 
-
     def subset_copy(self, row_index=None, column_index=None):
 
         if row_index is not None and column_index is not None:
@@ -855,6 +880,20 @@ class InferelatorData(object):
     def to_df(self):
         return self._adata.to_df()
 
+    def replace_data(self, new_data, new_gene_names=None, new_gene_metadata=None):
+
+        if new_gene_metadata is None and new_gene_names is not None:
+            new_gene_metadata = pd.DataFrame(index=new_gene_names)
+
+        self._adata = AnnData(
+            X=new_data,
+            dtype=new_data.dtype,
+            var=new_gene_metadata,
+            obs=self._adata.obs
+        )
+
+        gc.collect()
+
     @staticmethod
     def _make_idx_str(df):
         df.index = df.index.astype(str) if not pat.is_string_dtype(df.index.dtype) else df.index
